#ifndef EA_TEXTUR
#define EA_TEXTUR

#include "typen.h"
#include "vectors.h"
#include "grafik.h"

#define PERSPECTIVE 1
#define FULLPERSPECTIVE 0
/* set to 1 for slow but more realistic perspective projection  */
/* as opposed to faster polygon ("tetrangle") textures...       */
/* FULLPERSPECTIVE does projection for each pixel, while only   */
/* selecting PERSPECTIVE approximates as polygon projection.    */
/* Selecting neither option causes rectangular projection.      */
#define CLIPTEXTURE 1
/* set to 1 to clip the mapped texture - needed if your 3d      */
/* object is too big to be completely visible. Not much slower. */
#define RECTFACTOR 0.25
/* the zoom correction factor for non-perspective projection    */


#define scrdist 0.5 /* distance user from screen */
/* small values make textures smaller and emphasize perspective */
/* VEC yaxis  = { 0, 1, 0 }; */ /* y unit axis  */
/* VEC origin = { 0, 0, 0 }; */ /* space origin */

/* middle and scaling are used for bildpunkt() and bildtexture() */
#define cubesize ( ((W > H) ? H : W) * \
  ( (scrdist<2.0) ? (2.0/scrdist) : 1.0 ) )
/* should the scaling depend on scrdist ? */
#define skalx (1 * cubesize) /* X scaling */
#define skaly (1 * cubesize) /* Y scaling */

#define xm (W>>1) /* X middle */
#define ym (H>>1) /* Y middle */

typedef struct {
  VEC e0, ex0, ey0, ez0; /* origin, vectors */
} space3d;

typedef struct {
  VEC p0, px0, py0; /* origin, vectors */
} plane3d;

typedef struct {
  V_FP x;
  V_FP y;
} pointfp;

typedef struct {
  int x;
  int y;
} pointint;

typedef struct {
  V_FP x0,xx,xy, y0,yx,yy, z0,zx,zy; /* texture map status */
} texturemap;

void setview(const VEC * e0, const VEC *target, space3d * scr3d);
void bildpunkt(const space3d * screen,
  const VEC * iPt, pointint * oiPt, V_FP * dist);
void settextur(const space3d * screen,
  const plane3d * plane, texturemap * tmap);
void bildtextur(const pointint * iPt, const texturemap * tmap,
  pointint * oiPt);
void rotiere_y(int angle, const VEC * iPt, VEC * oPt);
void rotiere_z(int angle, const VEC * iPt, VEC * oPt);
void cubeplane(const space3d * scr3d, int oidx, int xidx, int yidx,
  VEC * pts3d, int resx, int resy,
  const EA_PIXEL * inbuf, int xorg, int yorg,
  EA_PIXEL * outbuf);
void cube(int phi1, int phi2, const EA_PIXEL * inbuf, EA_PIXEL * outbuf);

#endif

